import 'dotenv/config';

import { upload } from '../upload';
import { createLog } from '../log';
import { File } from '../file';
import { envString, envInt } from '../env';

import chokidar from 'chokidar';
import type { Logger } from 'winston';
import type { UploadResult, S3Config } from '../upload';
import type { FileInfo } from '../file';
import { SingleBar } from 'cli-progress';
import * as colors from 'ansi-colors';


const log : Logger = createLog('files');

function createConfig () : S3Config {
  const ENDPOINT : string = envString('OBJECT_ENDPOINT', '');
  const ACCESS_KEY : string = envString('OBJECT_ACCESS_KEY', '');
  const ACCESS_SECRET : string = envString('OBJECT_ACCESS_SECRET', '');
  const config : S3Config = {
    region: envString('OBJECT_REGION', 'us-east-1'),
    bucketName: envString('OBJECT_BUCKET', 'icebox'),
    expirationSeconds: envInt('OBJECT_EXPIRATION', 3600) //1 hour
  };

  if (ENDPOINT !== '') {
    config.endpoint = ENDPOINT;
    config.forcePathStyle = true;
  }

  if (ACCESS_KEY !== '' && ACCESS_SECRET !== '') {
    config.credentials = {
      accessKeyId : ACCESS_KEY,
      secretAccessKey : ACCESS_SECRET
    };
  }
  return config;
}

async function process (filePath : string) {
  const fileInfo : FileInfo = await File.info(filePath);
  const config : S3Config = createConfig();
  const progress : SingleBar = new SingleBar({
    format: `Printing [${fileInfo.filename}] | ${colors.cyan('{bar}')} | {percentage}% || ETA: {time}`,
    barCompleteChar: '\u2588',
    barIncompleteChar: '\u2591',
    hideCursor: true
  });
  if (!fileInfo.success) {
    log.error(`Error processing file info`, fileInfo.error);
    return;
  }

  console.dir(fileInfo);

	log.info(`Started upload: ${filePath}`);
  progress.start(100, 0, {
    time : 'N/A'
  });
	const result : UploadResult = await upload('test', filePath, config, (progressInfo : any) => {
    progress.update(Math.floor((progressInfo.loaded / progressInfo.total) * 100.0), { time : 'N/A'});
  });

  if (result.success) {
    progress.update(100, {
       time : 'Done'
    });
  }

  progress.stop();
	
	if (result.success) {
		log.info(`File ${filePath} uploaded successfully`);
		log.info(`Private URL: ${result.url}`);
	} else {
		log.error('Upload failed:', result.error);
	}
}

async function main() {
	/*chokidar.watch('./watch', { ignored: /(^|[/\\])\../ }).on('all', (event : any, path : string) => {
		log.info(`File ${path} changed with event type ${event}`);
	});*/
  process('/Users/matthewmcwilliams9/Desktop/m1554 hd optical printer test.mov');
}

main().catch(log.error);

/*
	const minioResult = await uploadFileToS3('/path/to/your/file.pdf', {
    region: 'us-east-1', // Region can be any string for MinIO
    endpoint: 'https://minio.your-domain.com',
    bucketName: 'your-minio-bucket',
    credentials: {
      accessKeyId: 'your-minio-access-key',
      secretAccessKey: 'your-minio-secret-key'
    },
    forcePathStyle: true, // Important for most S3-compatible services
    expirationSeconds: 3600
  });
  
  // Example 3: DigitalOcean Spaces
  const spacesResult = await uploadFileToS3('/path/to/your/file.pdf', {
    region: 'nyc3', // DigitalOcean datacenter region
    endpoint: 'https://nyc3.digitaloceanspaces.com',
    bucketName: 'your-space-name',
    credentials: {
      accessKeyId: 'your-spaces-key',
      secretAccessKey: 'your-spaces-secret'
    },
    forcePathStyle: true,
    expirationSeconds: 7200 // 2 hours
  });
  
  // Example 4: Wasabi
  const wasabiResult = await uploadFileToS3('/path/to/your/file.pdf', {
    region: 'us-east-1',
    endpoint: 'https://s3.wasabisys.com',
    bucketName: 'your-wasabi-bucket',
    credentials: {
      accessKeyId: 'your-wasabi-access-key',
      secretAccessKey: 'your-wasabi-secret-key'
    },
    forcePathStyle: true,
    expirationSeconds: 86400 // 24 hours
  });
*/