import { S3Client, PutObjectCommand, GetObjectCommand } from '@aws-sdk/client-s3';
import { getSignedUrl } from '@aws-sdk/s3-request-presigner';
import { ReadStream, createReadStream } from 'fs';
import { resolve, basename, extname } from 'path';
import { XhrHttpHandler } from '@aws-sdk/xhr-http-handler';
import { Upload } from '@aws-sdk/lib-storage';

// S3 configuration interface
interface S3Config {
    region: string;
    bucketName: string;
    expirationSeconds: number;
    endpoint?: string;
    credentials?: {
        accessKeyId: string;
        secretAccessKey: string;
    };
    forcePathStyle?: boolean;
}
// Function response interface
interface UploadResult {
    success: boolean;
    url?: string;
    key?: string;
    error?: string;
}

/**
 * Uploads a file to S3 and returns a private, time-limited URL to access it
 * @param filePath - Absolute or relative path to the file to upload
 * @param config - S3 configuration parameters
 * @returns Promise containing the result with URL if successful
 */
export async function upload (id: string, filePath: string, config: S3Config, progressCb : Function | null = null): Promise<UploadResult> {
    const fullPath : string = resolve(filePath);
    const fileName : string = basename(filePath);
    const extName  : string = extname(filePath);
    const key: string = `${id}/${fileName}`;

    if (!key) {
        return {
            success: false,
            error: 'Could not create key'
        };
    }
    
    let fileStream : ReadStream;
    
    try {
        fileStream = createReadStream(fullPath);
    } catch (err) {
        const error: Error = err as Error;
        console.error(error);
        return {
            success: false,
            error: `Error reading file: ${error.message}`
        };
    }
    
    const s3ClientOptions: Record<string, any> = {
        region: config.region//,
        //requestHandler: new XhrHttpHandler({})
    };

    if (config.endpoint) {
        s3ClientOptions.endpoint = config.endpoint;
    }

    if (config.credentials) {
        s3ClientOptions.credentials = config.credentials;
    }

    if (config.forcePathStyle !== undefined) {
        s3ClientOptions.forcePathStyle = config.forcePathStyle;
    }

    const s3Client: S3Client = new S3Client(s3ClientOptions);

    try {
        const upload : Upload = new Upload({
            client : s3Client,
            params: {
                Bucket : config.bucketName,
                Key : key,
                Body: fileStream
            }
        });

        upload.on('httpUploadProgress', (progress : any) => {
            if (progressCb !== null) {
                progressCb(progress);
            } else {
                console.log(`${progress.loaded}/${progress.total} [${Math.round((progress.loaded / progress.total) * 100.0)}%]`);
            }
        });

        await upload.done();
        
        const getCommand: GetObjectCommand = new GetObjectCommand({
            Bucket: config.bucketName,
            Key: key
        });
        
        const url : string = await getSignedUrl(s3Client, getCommand, {
            expiresIn: config.expirationSeconds
        });
        
        return {
            success: true,
            url,
            key
        };
    } catch (err) {
        const error : Error = err as Error;
        console.error(error);
        return {
            success: false,
            error: `Error uploading to S3: ${error.message}`
        };
    }
}

module.exports = { upload };
export type { UploadResult, S3Config };